from rdkit import Chem
from .AtomProperty import GetRelativeAtomicProperty

import numpy


class CalcMoreauBroto:
    def __init__(self, lag=1, tag='m'):
        self.lag = lag
        self.tag = tag

    def __call__(self, mol, **kwargs):
        n_atom = mol.GetNumAtoms()
        GetDistanceMatrix = Chem.GetDistanceMatrix(mol)
        res = 0.0
        for i in range(n_atom):
            for j in range(n_atom):
                if GetDistanceMatrix[i, j] == self.lag:
                    atom1 = mol.GetAtomWithIdx(i)
                    atom2 = mol.GetAtomWithIdx(j)
                    temp1 = GetRelativeAtomicProperty(element=atom1.GetSymbol(), propertyname=self.tag)
                    temp2 = GetRelativeAtomicProperty(element=atom2.GetSymbol(), propertyname=self.tag)
                    res += temp1 * temp2
                else:
                    res = res + 0.0
        return numpy.log(res / 2 + 1)


class CalcMorean:
    def __init__(self, lag=1, tag='m'):
        self.lag = lag
        self.tag = tag

    def __call__(self, mol, **kwargs):
        Natom = mol.GetNumAtoms()

        prolist = []
        for i in mol.GetAtoms():
            temp = GetRelativeAtomicProperty(i.GetSymbol(), propertyname=self.tag)
            prolist.append(temp)

        aveweight = sum(prolist) / Natom

        tempp = [numpy.square(x - aveweight) for x in prolist]

        GetDistanceMatrix = Chem.GetDistanceMatrix(mol)
        res = 0.0
        index = 0
        for i in range(Natom):
            for j in range(Natom):
                if GetDistanceMatrix[i, j] == self.lag:
                    atom1 = mol.GetAtomWithIdx(i)
                    atom2 = mol.GetAtomWithIdx(j)
                    temp1 = GetRelativeAtomicProperty(element=atom1.GetSymbol(), propertyname=self.tag)
                    temp2 = GetRelativeAtomicProperty(element=atom2.GetSymbol(), propertyname=self.tag)
                    res += (temp1 - aveweight) * (temp2 - aveweight)
                    index += 1
                else:
                    res = res + 0.0

        if sum(tempp) == 0 or index == 0:
            result = 0
        else:
            result = (res / index) / (sum(tempp) / Natom)

        return result


class CalcGerary:
    def __init__(self, lag=1, tag='m'):
        self.lag = lag
        self.tag = tag

    def __call__(self, mol, **kwargs):
        Natom = mol.GetNumAtoms()

        prolist = []
        for i in mol.GetAtoms():
            temp = GetRelativeAtomicProperty(i.GetSymbol(), propertyname=self.tag)
            prolist.append(temp)

        aveweight = sum(prolist) / Natom

        tempp = [numpy.square(x - aveweight) for x in prolist]

        GetDistanceMatrix = Chem.GetDistanceMatrix(mol)
        res = 0.0
        index = 0
        for i in range(Natom):
            for j in range(Natom):
                if GetDistanceMatrix[i, j] == self.lag:
                    atom1 = mol.GetAtomWithIdx(i)
                    atom2 = mol.GetAtomWithIdx(j)
                    temp1 = GetRelativeAtomicProperty(element=atom1.GetSymbol(), propertyname=self.tag)
                    temp2 = GetRelativeAtomicProperty(element=atom2.GetSymbol(), propertyname=self.tag)
                    res = res + numpy.square(temp1 - temp2)
                    index = index + 1
                else:
                    res = res + 0.0

        if sum(tempp) == 0 or index == 0:
            result = 0
        else:
            result = (res / index / 2) / (sum(tempp) / (Natom - 1))
        return result


from rdkit import Chem
from rdkit.Chem import GraphDescriptors as graph
from rdkit.Chem.EState import GetPrincipleQuantumNumber
import numpy as np


class WienerIdx:
    def __init__(self, is_average=False):
        self.is_average = is_average

    def __call__(self, m, **kwargs):
       amat = Chem.GetDistanceMatrix(m)
       res = sum(sum(amat))
       N = m.GetNumAtoms()
       if self.is_average:
           res = res / N / (N-1)
       return res / 2


def NumHarary(mol, **kwargs):
    dist = np.array(Chem.GetDistanceMatrix(mol), 'd')
    return 1.0 / 2 * (sum(1.0 / dist[dist != 0]))


def SchiultzIdx(mol, **kwargs):
    Distance = np.array(Chem.GetDistanceMatrix(mol), 'd')
    Adjacent = np.array(Chem.GetAdjacencyMatrix(mol), 'd')
    VertexDegree = sum(Adjacent)
    return sum(np.dot((Distance + Adjacent), VertexDegree))


def GraphDistIdx(mol, **kwargs):
    Distance = Chem.GetDistanceMatrix(mol)
    n = int(Distance.max())
    res = 0.0
    for i in range(n):
        # print Distance==i+1
        temp = 1. / 2 * sum(sum(Distance == i + 1))
        # print temp
        res = res + temp ** 2
    return np.log10(res)


def CalcPlatt(mol, **kwargs):
    cc = [x.GetBeginAtom().GetDegree() + x.GetEndAtom().GetDegree() - 2 for x in mol.GetBonds()]
    return sum(cc)


def XuIdx(mol, **kwargs):
    nAT = mol.GetNumAtoms()
    deltas = [x.GetDegree() for x in mol.GetAtoms()]
    Distance = Chem.GetDistanceMatrix(mol)
    sigma = np.sum(Distance, axis=1)
    temp1 = 0.0
    temp2 = 0.0
    for i in range(nAT):
        temp1 = temp1 + deltas[i] * ((sigma[i]) ** 2)
        temp2 = temp2 + deltas[i] * (sigma[i])
    Xu = np.sqrt(nAT) * np.log(temp1 / temp2)
    return Xu


def NumPolarity(mol, **kwargs):
    dist = Chem.GetDistanceMatrix(mol)
    res = 1. / 2 * sum(sum(dist == 3))
    return res


def PoglianiIdx(mol, **kwargs):
    res = 0.0
    for atom in mol.GetAtoms():
        n = atom.GetAtomicNum()
        nV = Chem.GetPeriodicTable().GetNOuterElecs(n)
        mP = GetPrincipleQuantumNumber(n)
        res = res + (nV + 0.0) / mP
    return res


def IpcIdx(mol, **kwargs):
    return np.log10(graph.Ipc(mol))


def BertzCT(mol, **kwargs):
    return np.log10(graph.BertzCT(mol))


def GutmanTopo(mol, **kwargs):
    nAT = mol.GetNumAtoms()
    deltas = [x.GetDegree() for x in mol.GetAtoms()]
    dist = Chem.GetDistanceMatrix(mol)
    res = 0.0
    for i in range(nAT):
        for j in range(i + 1, nAT):
            res = res + deltas[i] * deltas[j] * dist[i, j]

    return np.log10(res)


def Zagreb1(mol, **kwargs):
    deltas = [x.GetDegree() for x in mol.GetAtoms()]
    return sum(np.array(deltas) ** 2)


def Zagreb2(mol, **kwargs):
    ke = [x.GetBeginAtom().GetDegree() * x.GetEndAtom().GetDegree() for x in mol.GetBonds()]
    return sum(ke)


def MZagreb1(mol, **kwargs):
    deltas = [x.GetDegree() for x in mol.GetAtoms()]
    while 0 in deltas:
        deltas.remove(0)
    deltas = np.array(deltas, 'd')
    res = sum((1. / deltas) ** 2)
    return res


def MZagreb2(mol, **kwargs):
    cc = [x.GetBeginAtom().GetDegree() * x.GetEndAtom().GetDegree() for x in mol.GetBonds()]
    while 0 in cc:
        cc.remove(0)
    cc = np.array(cc, 'd')
    res = sum((1. / cc) ** 2)
    return res


def Quadratic(mol, **kwargs):
    M = Zagreb1(mol)
    N = mol.GetNumAtoms()
    return 3 - 2 * N + M / 2.0


def Diameter(mol, **kwargs):
    dist = Chem.GetDistanceMatrix(mol)
    return dist.max()


def Radius(mol, **kwargs):
    Distance = Chem.GetDistanceMatrix(mol)
    temp = []
    for i in Distance:
        temp.append(max(i))
    return min(temp)


def Petitjean(mol, **kwargs):
    diameter =Diameter(mol)
    radius = Radius(mol)
    return 1 - radius / float(diameter)


def SimpleTopovIdx(mol, **kwargs):
    deltas = graph._hkDeltas(mol, skipHs=0)
    while 0 in deltas:
        deltas.remove(0)
    deltas = np.array(deltas, 'd')

    res = np.prod(deltas)

    return np.log(res)


def HarmonicTopovIdx(mol, **kwargs):
    deltas = graph._hkDeltas(mol, skipHs=0)
    while 0 in deltas:
        deltas.remove(0)
    deltas = np.array(deltas, 'd')
    nAtoms = mol.GetNumAtoms()

    res = nAtoms / sum(1. / deltas)

    return res


def GeometricTopovIdx(mol, **kwargs):
    nAtoms = mol.GetNumAtoms()
    deltas = graph._hkDeltas(mol, skipHs=0)
    while 0 in deltas:
        deltas.remove(0)
    deltas = np.array(deltas, 'd')

    temp = np.prod(deltas)
    res = np.power(temp, 1. / nAtoms)

    return res


def ArithmeticTopoIdx(mol, **kwargs):
    nAtoms = mol.GetNumAtoms()
    nBonds = mol.GetNumBonds()
    res = 2. * nBonds / nAtoms
    return res



from .autocor import *
from .charge import *
from .connectivity import *
from .constitution import *
from .estate import *
from .fingerprint import *
from .topology import *
from .kappa import *
from .property import *
from .basak import *
from rdkit.Chem import Descriptors as desc
import pandas as pd

ATSm1 = CalcMoreauBroto(lag=1, tag='m')
ATSm2 = CalcMoreauBroto(lag=2, tag='m')
ATSm3 = CalcMoreauBroto(lag=3, tag='m')
ATSm4 = CalcMoreauBroto(lag=4, tag='m')
ATSm5 = CalcMoreauBroto(lag=5, tag='m')
ATSm6 = CalcMoreauBroto(lag=6, tag='m')
ATSm7 = CalcMoreauBroto(lag=7, tag='m')
ATSm8 = CalcMoreauBroto(lag=8, tag='m')
ATSv1 = CalcMoreauBroto(lag=1, tag='V')
ATSv2 = CalcMoreauBroto(lag=2, tag='V')
ATSv3 = CalcMoreauBroto(lag=3, tag='V')
ATSv4 = CalcMoreauBroto(lag=4, tag='V')
ATSv5 = CalcMoreauBroto(lag=5, tag='V')
ATSv6 = CalcMoreauBroto(lag=6, tag='V')
ATSv7 = CalcMoreauBroto(lag=7, tag='V')
ATSv8 = CalcMoreauBroto(lag=8, tag='V')
ATSe1 = CalcMoreauBroto(lag=1, tag='En')
ATSe2 = CalcMoreauBroto(lag=2, tag='En')
ATSe3 = CalcMoreauBroto(lag=3, tag='En')
ATSe4 = CalcMoreauBroto(lag=4, tag='En')
ATSe5 = CalcMoreauBroto(lag=5, tag='En')
ATSe6 = CalcMoreauBroto(lag=6, tag='En')
ATSe7 = CalcMoreauBroto(lag=7, tag='En')
ATSe8 = CalcMoreauBroto(lag=8, tag='En')
ATSp1 = CalcMoreauBroto(lag=1, tag='alpha')
ATSp2 = CalcMoreauBroto(lag=2, tag='alpha')
ATSp3 = CalcMoreauBroto(lag=3, tag='alpha')
ATSp4 = CalcMoreauBroto(lag=4, tag='alpha')
ATSp5 = CalcMoreauBroto(lag=5, tag='alpha')
ATSp6 = CalcMoreauBroto(lag=6, tag='alpha')
ATSp7 = CalcMoreauBroto(lag=7, tag='alpha')
ATSp8 = CalcMoreauBroto(lag=8, tag='alpha')

MATSm1 = CalcMorean(lag=1, tag='m')
MATSm2 = CalcMorean(lag=2, tag='m')
MATSm3 = CalcMorean(lag=3, tag='m')
MATSm4 = CalcMorean(lag=4, tag='m')
MATSm5 = CalcMorean(lag=5, tag='m')
MATSm6 = CalcMorean(lag=6, tag='m')
MATSm7 = CalcMorean(lag=7, tag='m')
MATSm8 = CalcMorean(lag=8, tag='m')
MATSv1 = CalcMorean(lag=1, tag='V')
MATSv2 = CalcMorean(lag=2, tag='V')
MATSv3 = CalcMorean(lag=3, tag='V')
MATSv4 = CalcMorean(lag=4, tag='V')
MATSv5 = CalcMorean(lag=5, tag='V')
MATSv6 = CalcMorean(lag=6, tag='V')
MATSv7 = CalcMorean(lag=7, tag='V')
MATSv8 = CalcMorean(lag=8, tag='V')
MATSe1 = CalcMorean(lag=1, tag='En')
MATSe2 = CalcMorean(lag=2, tag='En')
MATSe3 = CalcMorean(lag=3, tag='En')
MATSe4 = CalcMorean(lag=4, tag='En')
MATSe5 = CalcMorean(lag=5, tag='En')
MATSe6 = CalcMorean(lag=6, tag='En')
MATSe7 = CalcMorean(lag=7, tag='En')
MATSe8 = CalcMorean(lag=8, tag='En')
MATSp1 = CalcMorean(lag=1, tag='alpha')
MATSp2 = CalcMorean(lag=2, tag='alpha')
MATSp3 = CalcMorean(lag=3, tag='alpha')
MATSp4 = CalcMorean(lag=4, tag='alpha')
MATSp5 = CalcMorean(lag=5, tag='alpha')
MATSp6 = CalcMorean(lag=6, tag='alpha')
MATSp7 = CalcMorean(lag=7, tag='alpha')
MATSp8 = CalcMorean(lag=8, tag='alpha')

GATSm1 = CalcGerary(lag=1, tag='m')
GATSm2 = CalcGerary(lag=2, tag='m')
GATSm3 = CalcGerary(lag=3, tag='m')
GATSm4 = CalcGerary(lag=4, tag='m')
GATSm5 = CalcGerary(lag=5, tag='m')
GATSm6 = CalcGerary(lag=6, tag='m')
GATSm7 = CalcGerary(lag=7, tag='m')
GATSm8 = CalcGerary(lag=8, tag='m')
GATSv1 = CalcGerary(lag=1, tag='V')
GATSv2 = CalcGerary(lag=2, tag='V')
GATSv3 = CalcGerary(lag=3, tag='V')
GATSv4 = CalcGerary(lag=4, tag='V')
GATSv5 = CalcGerary(lag=5, tag='V')
GATSv6 = CalcGerary(lag=6, tag='V')
GATSv7 = CalcGerary(lag=7, tag='V')
GATSv8 = CalcGerary(lag=8, tag='V')
GATSe1 = CalcGerary(lag=1, tag='En')
GATSe2 = CalcGerary(lag=2, tag='En')
GATSe3 = CalcGerary(lag=3, tag='En')
GATSe4 = CalcGerary(lag=4, tag='En')
GATSe5 = CalcGerary(lag=5, tag='En')
GATSe6 = CalcGerary(lag=6, tag='En')
GATSe7 = CalcGerary(lag=7, tag='En')
GATSe8 = CalcGerary(lag=8, tag='En')
GATSp1 = CalcGerary(lag=1, tag='alpha')
GATSp2 = CalcGerary(lag=2, tag='alpha')
GATSp3 = CalcGerary(lag=3, tag='alpha')
GATSp4 = CalcGerary(lag=4, tag='alpha')
GATSp5 = CalcGerary(lag=5, tag='alpha')
GATSp6 = CalcGerary(lag=6, tag='alpha')
GATSp7 = CalcGerary(lag=7, tag='alpha')
GATSp8 = CalcGerary(lag=8, tag='alpha')

SPP = CalcSubmolPolarityPara
LDI = CalcLocalDipoleIndex
Rnc = CalcElementCharge(0, reln_sum)
Rpc = CalcElementCharge(0, relp_sum)
Mac = CalcElementCharge(0, abs_mean)
Tac = CalcElementCharge(0, abs_sum)
Mnc = CalcElementCharge(0, n_mean)
Tnc = CalcElementCharge(0, n_sum)
Mpc = CalcElementCharge(0, p_mean)
Tpc = CalcElementCharge(0, p_sum)
Qass = CalcElementCharge(0, sqsum)
QOss = CalcElementCharge(6, sqsum)
QNss = CalcElementCharge(7, sqsum)
QCss = CalcElementCharge(8, sqsum)
QHss = CalcElementCharge(1, sqsum)
Qmin = CalcElementCharge(0, np.min)
QOmin = CalcElementCharge(6, np.min)
QNmin = CalcElementCharge(7, np.min)
QCmin = CalcElementCharge(8, np.min)
QHmin = CalcElementCharge(1, np.min)
Qmax = CalcElementCharge(0, np.max)
QOmax = CalcElementCharge(6, np.max)
QNmax = CalcElementCharge(7, np.max)
QCmax = CalcElementCharge(8, np.max)
QHmax = CalcElementCharge(1, np.max)

mChi1 = MeanRandic
Chi0 = Chinp(n_path=0)
Chi1 = Chinp(n_path=0)
Chi2 = Chinp(n_path=2)
Chi3 = Chinp(n_path=3)
Chi4 = Chinp(n_path=4)
Chi5 = Chinp(n_path=5)
Chi6 = Chinp(n_path=6)
Chi7 = Chinp(n_path=7)
Chi8 = Chinp(n_path=8)
Chi9 = Chinp(n_path=9)
Chi10 = Chinp(n_path=10)
Chi3c = Chinc(tag='3', is_hk=False)
Chi4c = Chinc(tag='4', is_hk=False)
Chi4pc = Chinc(tag='4p', is_hk=False)
Chi3ch = Chinch(n_cycle=3)
Chi4ch = Chinch(n_cycle=4)
Chi5ch = Chinch(n_cycle=5)
Chi6ch = Chinch(n_cycle=6)
Chiv0 = Chivnp(n_path=0)
Chiv1 = Chivnp(n_path=1)
Chiv2 = Chivnp(n_path=2)
Chiv3 = Chivnp(n_path=3)
Chiv4 = Chivnp(n_path=4)
Chiv5 = Chivnp(n_path=5)
Chiv6 = Chivnp(n_path=6)
Chiv7 = Chivnp(n_path=7)
Chiv8 = Chivnp(n_path=8)
Chiv9 = Chivnp(n_path=9)
Chiv10 = Chivnp(n_path=10)
dchi0 = DeltaChi(Chivnp(0), Chinp(0))
dchi1 = DeltaChi(Chivnp(1), Chinp(1))
dchi2 = DeltaChi(Chivnp(2), Chinp(2))
dchi3 = DeltaChi(Chivnp(3), Chinp(3))
dchi4 = DeltaChi(Chivnp(4), Chinp(4))
Chiv3c = Chinc(tag='3', is_hk=True)
Chiv4c = Chinc(tag='4', is_hk=True)
Chiv4pc = Chinc(tag='4p', is_hk=True)
Chiv3ch = Chivnch(n_cycle=3)
Chiv4ch = Chivnch(n_cycle=4)
Chiv5ch = Chivnch(n_cycle=5)
Chiv6ch = Chivnch(n_cycle=6)
knotpv = DeltaChi(Chinc(tag='3', is_hk=True), Chinc(tag='4p', is_hk=True))
knotp = DeltaChi(Chinc(tag='3', is_hk=False), Chinc(tag='4p', is_hk=False))

Weight = desc.MolWt
nhyd = FragCounter('[H]')
nhal = FragCounter('[F,Cl,Br,I]')
nhet = AllChem.CalcNumHeteroatoms
nring = AllChem.CalcNumRings
nrot = AllChem.CalcNumRotatableBonds
ndonr = AllChem.CalcNumHBD
naccr = AllChem.CalcNumHBA

nhev = FragCounter('[!H]')
ncof = FragCounter('F')
ncocl = FragCounter('Cl')
ncobr = FragCounter('Br')
ncoi = FragCounter('I')
ncarb = FragCounter('C')
nphos = FragCounter('P')
nsulph = FragCounter('S')
noxy = FragCounter('O')
nnitro = FragCounter('N')
nsb = FragCounter('[*]-[*]')
ndb = FragCounter('[*]=[*]')
ntb = FragCounter('[*]#[*]')
naro = FragCounter('[*]:[*]')
nta = TotalAtom()
AWeight = TotalAtom(is_weight=True)
PC1 = PathsOfLengthN(1)
PC2 = PathsOfLengthN(2)
PC3 = PathsOfLengthN(3)
PC4 = PathsOfLengthN(4)
PC5 = PathsOfLengthN(5)
PC6 = PathsOfLengthN(6)

value = CalcEstateValue
max = CalcMaxAtomTypeEState
min = CalcMinAtomTypeEState
Shev = CalcHeavyAtomEState
Scar = CalcCAtomEState
Shal = CalculateHalogenEState
Shet = CalculateHeteroEState
Save = CalcAverageEState
Smax = CalcMaxEState
Smin = CalcMinEState
DS = CalcDiffMaxMinEState

slogPVSA = MolSurf.SlogP_VSA_
MRVSA = MolSurf.SMR_VSA_
PEOEVSA = MolSurf.PEOE_VSA_
EstateVSA = EVSA.EState_VSA_
VSAEstate = EVSA.VSA_EState_

topological = CalcDaylightFingerprint
Estate = CalcEstateFingerprint
atompairs = CalcAtomPairsFingerprint
torsions = CalculateTopologicalTorsionFingerprint
# morgan = CalculateMorganFingerprint
ECFP4 = CalcMorganFingerprint(radius=2, useFeatures=False)
ECFP6 = CalcMorganFingerprint(radius=3, useFeatures=False)
FCFP4 = CalcMorganFingerprint(radius=2, useFeatures=True)
FCFP6 = CalcMorganFingerprint(radius=3, useFeatures=True)
MACCS = CalculateMACCSFingerprint

W = WienerIdx()
AW = WienerIdx(is_average=True)
J = graph.BalabanJ
Thara = NumHarary
Tsch = SchiultzIdx
Tigdi = GraphDistIdx
Platt = CalcPlatt
Xu = XuIdx
Pol = NumPolarity
Dz = PoglianiIdx
Ipc = IpcIdx
BertzCT = BertzCT
GMTI = GutmanTopo
ZM1 = Zagreb1
ZM2 = Zagreb2
MZM1 = MZagreb1
MZM2 = MZagreb2
Qindex = Quadratic
diametert = Diameter
radiust = Radius
petitjeant = Petitjean
Sito = SimpleTopovIdx
Hato = HarmonicTopovIdx
Geto = GeometricTopovIdx
Arto = ArithmeticTopoIdx

kappa1 = CalcKappa(is_alpha=False, n_bond=1)
kappa2 = CalcKappa(is_alpha=False, n_bond=2)
kappa3 = CalcKappa(is_alpha=False, n_bond=3)
kappam1 = CalcKappa(is_alpha=True, n_bond=1)
kappam2 = CalcKappa(is_alpha=True, n_bond=2)
kappam3 = CalcKappa(is_alpha=True, n_bond=3)
phi = Flexibility

LogP = Crippen.MolLogP
MR = Crippen.MolMR
LabuteASA = MS.pyLabuteASA
TPSA = MS.TPSA
Hy = CalculateHydrophilicityFactor
UI = CalculateUnsaturationIndex

# Basak
CIC0 = CalcBasakCIC0
CIC1 = CalcBasakCICn(num_path=2)
CIC2 = CalcBasakCICn(num_path=3)
CIC3 = CalcBasakCICn(num_path=4)
CIC4 = CalcBasakCICn(num_path=5)
CIC5 = CalcBasakCICn(num_path=6)
CIC6 = CalcBasakCICn(num_path=7)
SIC0 = CalcBasakSIC0
SIC1 = CalcBasakSICn(num_path=2)
SIC2 = CalcBasakSICn(num_path=3)
SIC3 = CalcBasakSICn(num_path=4)
SIC4 = CalcBasakSICn(num_path=5)
SIC5 = CalcBasakSICn(num_path=6)
SIC6 = CalcBasakSICn(num_path=7)
IC0 = CalcBasakIC0
IC1 = CalcBasakICn(num_path=2)
IC2 = CalcBasakICn(num_path=3)
IC3 = CalcBasakICn(num_path=4)
IC4 = CalcBasakICn(num_path=5)
IC5 = CalcBasakICn(num_path=6)
IC6 = CalcBasakICn(num_path=7)

# Burden
from .burden import *
bcutp = CalcBurden(label='alpha')
bcute = CalcBurden(label='En')
bcutv = CalcBurden(label='V')
bcutm = CalcBurden(label='m')

# Pharmacophore
from .cats import *



from rdkit.Chem.EState import Fingerprinter as ESFP
from rdkit import Chem
from rdkit.Chem.EState import GetPrincipleQuantumNumber

from . import AtomTypes as ATEstate
import numpy as np


def _CalculateEState(mol, skipH=1):
    """
    #################################################################
    Get the EState value of each atom in a molecule
    #################################################################
    """
    mol = Chem.AddHs(mol)
    if skipH == 1:
        mol = Chem.RemoveHs(mol)
    tb1 = Chem.GetPeriodicTable()
    nAtoms = mol.GetNumAtoms()
    Is = np.zeros(nAtoms, np.float)
    for i in range(nAtoms):
        at = mol.GetAtomWithIdx(i)
        atNum = at.GetAtomicNum()
        d = at.GetDegree()
        if d > 0:
            h = at.GetTotalNumHs()
            dv = tb1.GetNOuterElecs(atNum) - h
            # dv=np.array(_AtomHKDeltas(at),'d')
            N = GetPrincipleQuantumNumber(atNum)
            Is[i] = (4.0 / (N * N) * dv + 1) / d
    dists = Chem.GetDistanceMatrix(mol, useBO=0, useAtomWts=0)
    dists += 1
    accum = np.zeros(nAtoms, np.float)
    for i in range(nAtoms):
        for j in range(i + 1, nAtoms):
            p = dists[i, j]
            if p < 1e6:
                temp = (Is[i] - Is[j]) / (p * p)
                accum[i] += temp
                accum[j] -= temp
    res = accum + Is
    return res


def CalcHeavyAtomEState(mol, **kwargs):
    """
    #################################################################
    The sum of the EState indices over all non-hydrogen atoms

    -->Shev
    #################################################################
    """
    return sum(_CalculateEState(mol))


def _CalculateAtomEState(mol, AtomicNum=6):
    """
    #################################################################
    **Internal used only**

    The sum of the EState indices over all atoms
    #################################################################
    """
    nAtoms = mol.GetNumAtoms()
    Is = np.zeros(nAtoms, np.float)
    Estate = _CalculateEState(mol)

    for i in range(nAtoms):
        at = mol.GetAtomWithIdx(i)
        atNum = at.GetAtomicNum()
        if atNum == AtomicNum:
            Is[i] = Estate[i]
    res = sum(Is)

    return res


def CalcCAtomEState(mol, **kwargs):
    """
    #################################################################
    The sum of the EState indices over all C atoms

    -->Scar
    #################################################################
    """
    return _CalculateAtomEState(mol, AtomicNum=6)


def CalculateHalogenEState(mol, **kwargs):
    """
    #################################################################
    The sum of the EState indices over all Halogen atoms

    -->Shal
    #################################################################
    """

    Nf = _CalculateAtomEState(mol, AtomicNum=9)
    Ncl = _CalculateAtomEState(mol, AtomicNum=17)
    Nbr = _CalculateAtomEState(mol, AtomicNum=35)
    Ni = _CalculateAtomEState(mol, AtomicNum=53)

    return Nf + Ncl + Nbr + Ni


def CalculateHeteroEState(mol, **kwargs):
    """
    #################################################################
    The sum of the EState indices over all hetero atoms

    -->Shet
    #################################################################
    """

    Ntotal = sum(_CalculateEState(mol))
    NC = _CalculateAtomEState(mol, AtomicNum=6)
    NH = _CalculateAtomEState(mol, AtomicNum=1)

    return Ntotal - NC - NH


def CalcAverageEState(mol, **kwargs):
    """
    #################################################################
    The sum of the EState indices over all non-hydrogen atoms

    divided by the number of non-hydrogen atoms.

    -->Save
    #################################################################
    """
    return np.mean(_CalculateEState(mol))


def CalcMaxEState(mol, **kwargs):
    """
    #################################################################
    Obtain the maximal Estate value in all atoms

    -->Smax
    #################################################################
    """
    return max(_CalculateEState(mol))


def CalcMinEState(mol, **kwargs):
    """
    #################################################################
    Obtain the minimal Estate value in all atoms

    -->Smin
    #################################################################
    """

    return min(_CalculateEState(mol))


def CalcDiffMaxMinEState(mol, **kwargs):
    """
    #################################################################
    The difference between Smax and Smin

    -->DS
    #################################################################
    """
    return max(_CalculateEState(mol)) - min(_CalculateEState(mol))


def CalcAtomTypeEState(mol, **kwargs):
    """
    #################################################################
    Calculation of sum of E-State value of specified atom type

    res---->list type
    #################################################################
    """
    AT = ATEstate.GetAtomLabel(mol)
    Estate = _CalculateEState(mol)
    res = []
    for i in AT:
        if i == []:
            res.append(0)
        else:
            res.append(sum([Estate[k] for k in i]))
    return res


def CalcEstateFingerprint(mol, **kwargs):
    """
    #################################################################
    The Calculation of EState Fingerprints.

    It is the number of times each possible atom type is hit.

    Usage:

        result=CalculateEstateFingerprint(mol)

        Input: mol is a molecule object.

        Output: result is a dict form containing 79 estate fragments.
    #################################################################
    """
    res = ESFP.FingerprintMol(mol)[0]
    return res


def CalcEstateValue(mol, **kwargs):
    """
    #################################################################
    The Calculate of EState Values.

    It is the sum of the Estate indices for atoms of each type.

    Usage:

        result=CalculateEstateValue(mol)

        Input: mol is a molecule object.

        Output: result is a dict form containing 79 estate values.
    #################################################################
    """
    res = ESFP.FingerprintMol(mol)[1]
    return res


def CalcMaxAtomTypeEState(mol, **kwargs):
    """
    #################################################################
    Calculation of maximum of E-State value of specified atom type

    res---->dict type

    Usage:

        result=CalculateMaxAtomTypeEState(mol)

        Input: mol is a molecule object.

        Output: result is a dict form containing 79 max estate values.
    #################################################################
    """
    AT = ATEstate.GetAtomLabel(mol)
    Estate = _CalculateEState(mol)
    res = []
    for i in AT:
        if i == []:
            res.append(0)
        else:
            res.append(max([Estate[k] for k in i]))
    return res


def CalcMinAtomTypeEState(mol):
    """
    #################################################################
    Calculation of minimum of E-State value of specified atom type

    res---->dict type

    Usage:

        result=CalculateMinAtomTypeEState(mol)

        Input: mol is a molecule object.

        Output: result is a dict form containing 79 min estate values.
    #################################################################
    """
    AT = ATEstate.GetAtomLabel(mol)
    Estate = _CalculateEState(mol)
    res = []
    for i in AT:
        if i == []:
            res.append(0)
        else:
            res.append(min([Estate[k] for k in i]))
    return res




import numpy
import numpy.linalg
from rdkit import Chem
from .AtomProperty import GetRelativeAtomicProperty


def _GetBurdenMatrix(mol, propertylabel="m"):
    mol = Chem.AddHs(mol)
    Natom = mol.GetNumAtoms()

    AdMatrix = Chem.GetAdjacencyMatrix(mol)
    bondindex = numpy.argwhere(AdMatrix)
    AdMatrix1 = numpy.array(AdMatrix, dtype=numpy.float32)

    # The diagonal elements of B, Bii, are either given by
    # the carbon normalized atomic mass,
    # van der Waals volume, Sanderson electronegativity,
    # and polarizability of atom i.

    for i in range(Natom):
        atom = mol.GetAtomWithIdx(i)
        temp = GetRelativeAtomicProperty(
            element=atom.GetSymbol(), propertyname=propertylabel
        )
        AdMatrix1[i, i] = round(temp, 3)

    # The element of B connecting atoms i and j, Bij,
    # is equal to the square root of the bond
    # order between atoms i and j.

    for i in bondindex:
        bond = mol.GetBondBetweenAtoms(int(i[0]), int(i[1]))
        if bond.GetBondType().name == "SINGLE":
            AdMatrix1[i[0], i[1]] = round(numpy.sqrt(1), 3)
        if bond.GetBondType().name == "DOUBLE":
            AdMatrix1[i[0], i[1]] = round(numpy.sqrt(2), 3)
        if bond.GetBondType().name == "TRIPLE":
            AdMatrix1[i[0], i[1]] = round(numpy.sqrt(3), 3)
        if bond.GetBondType().name == "AROMATIC":
            AdMatrix1[i[0], i[1]] = round(numpy.sqrt(1.5), 3)

    ##All other elements of B (corresponding non bonded
    # atom pairs) are set to 0.001
    bondnonindex = numpy.argwhere(AdMatrix == 0)

    for i in bondnonindex:
        if i[0] != i[1]:

            AdMatrix1[i[0], i[1]] = 0.001

    return numpy.real(numpy.linalg.eigvals(AdMatrix1))


class CalcBurden:
    def __init__(self, label='m'):
        self.label = label

    def __call__(self, mol, **kwargs):
        temp = _GetBurdenMatrix(mol, propertylabel=self.label)
        temp1 = numpy.sort(temp[temp >= 0])
        temp2 = numpy.sort(numpy.abs(temp[temp < 0]))

        if len(temp1) < 8:
            temp1 = numpy.concatenate((numpy.zeros(8), temp1))
        if len(temp2) < 8:
            temp2 = numpy.concatenate((numpy.zeros(8), temp2))
        bcutvalue = numpy.concatenate((temp2[-8:], temp1[-8:]))
        return bcutvalue







from rdkit.Chem import MolSurf
from rdkit.Chem.EState import EState_VSA as EVSA
from rdkit.Chem.Fingerprints import FingerprintMols
from rdkit.Chem import MACCSkeys
from rdkit.Chem import AllChem
from .estate import CalcEstateFingerprint as EstateFingerprint
from rdkit.Chem.AtomPairs import Pairs
from rdkit.Chem.AtomPairs import Torsions
from rdkit import DataStructs
import numpy as np


def CalcDaylightFingerprint(mol, **kwargs):
    """
    #################################################################
    Calculate Daylight-like fingerprint or topological fingerprint

    (128 bits).

    Usage:

        result=CalculateDaylightFingerprint(mol)

        Input: mol is a molecule object.

        Output: result is a tuple form. The first is the number of

        fingerprints. The second is a dict form whose keys are the

        position which this molecule has some substructure. The third

        is the DataStructs which is used for calculating the similarity.
    #################################################################
    """
    res = np.zeros(128)
    # NumFinger = 128
    bv = FingerprintMols.FingerprintMol(mol)
    DataStructs.ConvertToNumpyArray(bv, res)

    return res


def CalculateMACCSFingerprint(mol, **kwargs):
    """
    #################################################################
    Calculate MACCS keys (166 bits).

    Usage:

        result=CalculateMACCSFingerprint(mol)

        Input: mol is a molecule object.

        Output: result is a tuple form. The first is the number of

        fingerprints. The second is a dict form whose keys are the

        position which this molecule has some substructure. The third

        is the DataStructs which is used for calculating the similarity.
    #################################################################
    """
    res = np.zeros(166)
    bv = MACCSkeys.GenMACCSKeys(mol)
    DataStructs.ConvertToNumpyArray(bv, res)

    return res


def CalcEstateFingerprint(mol, **kwargs):
    """
    #################################################################
    Calculate E-state fingerprints (79 bits).

    Usage:

        result=CalculateEstateFingerprint(mol)

        Input: mol is a molecule object.

        Output: result is a tuple form. The first is the number of

        fingerprints. The second is a dict form whose keys are the

        position which this molecule has some substructure. The third

        is the DataStructs which is used for calculating the similarity.
    #################################################################
    """
    NumFinger = 79
    res = {}
    temp = EstateFingerprint(mol, **kwargs)
    for i in temp:
        if temp[i] > 0:
            res[i[7:]] = 1

    return temp


def CalcAtomPairsFingerprint(mol, **kwargs):
    """
    #################################################################
    Calculate atom pairs fingerprints

    Usage:

        result=CalculateAtomPairsFingerprint(mol)

        Input: mol is a molecule object.

        Output: result is a tuple form. The first is the number of

        fingerprints. The second is a dict form whose keys are the

        position which this molecule has some substructure. The third

        is the DataStructs which is used for calculating the similarity.
    #################################################################
    """
    res = np.zeros(1)
    bv = Pairs.GetAtomPairFingerprint(mol)
    DataStructs.ConvertToNumpyArray(bv, res)
    return res


def CalculateTopologicalTorsionFingerprint(mol, **kwargs):
    """
    #################################################################
    Calculate Topological Torsion Fingerprints

    Usage:

        result=CalculateTopologicalTorsionFingerprint(mol)

        Input: mol is a molecule object.

        Output: result is a tuple form. The first is the number of

        fingerprints. The second is a dict form whose keys are the

        position which this molecule has some substructure. The third

        is the DataStructs which is used for calculating the similarity.
    #################################################################
    """
    res = Torsions.GetTopologicalTorsionFingerprint(mol)

    return res.GetLength(), res.GetNonzeroElements(), res


def CalculateMorganFingerprint(mol, **kwargs):
    """
    #################################################################
    Calculate Morgan

    Usage:

        result=CalculateMorganFingerprint(mol)

        Input: mol is a molecule object.

        radius is a radius.

        Output: result is a tuple form. The first is the number of

        fingerprints. The second is a dict form whose keys are the

        position which this molecule has some substructure. The third

        is the DataStructs which is used for calculating the similarity.
    #################################################################
    """
    res = AllChem.GetMorganFingerprint(mol, 2)

    return res.GetLength(), res.GetNonzeroElements(), res


def CalculateSimilarity(fp1, fp2, similarity="Tanimoto"):
    """
    #################################################################
    Calculate similarity between two molecules.

    Usage:

        result=CalculateSimilarity(fp1,fp2)

        Input: fp1 and fp2 are two DataStructs.

        Output: result is a similarity value.
    #################################################################
    """
    temp = DataStructs.similarityFunctions
    for i in temp:
        if similarity in i[0]:
            similarityfunction = i[1]
        else:
            similarityfunction = temp[0][1]

    res = similarityfunction(fp1, fp2)
    return round(res, 3)


import scipy
from rdkit import Chem

PPP = {
    "D": ["[OH]", "[#7H,#7H2]"],
    "A": ["[O]", "[#7H0]"],
    "P": ["[*+]", "[#7H2]"],
    "N": ["[*-]", "[C&$(C(=O)O)]", "[P&$(P(=O)O)]", "[S&$(S(=O)O)]"],
    "L": ["[Cl,Br,I]", "[S;D2;$(S(C)(C))]"],
}


def MatchAtomType(IndexList, AtomTypeDict):
    """
    #################################################################
    Mapping two atoms with a certain distance into their atom types
    such as AA,AL, DP,LD etc.
    The result is a list format.
    #################################################################
    """
    First = []
    Second = []
    for i in AtomTypeDict:
        if IndexList[0] in AtomTypeDict[i]:
            First.append(i)
        if IndexList[1] in AtomTypeDict[i]:
            Second.append(i)

    temp = []
    for i in First:
        for j in Second:
            temp.append(i + j)

    temp1 = []
    for i in temp:
        if i in ["AD", "PD", "ND", "LD", "PA", "NA", "LA", "NP", "LN", "LP"]:
            temp1.append(i[1] + i[0])
        else:
            temp1.append(i)

    res = []
    for i in temp1:
        if i not in res:
            res.append(i)

    return res


###################################
def ContructLFromGraphSearch(mol):
    """
    #################################################################
    The last lipophilic pattern on page 55 of the book is realized as a graph
    search and not as a SMARTS search.
    "L" carbon atom adjacent only to carbon atoms.
    The result is a list format.
    #################################################################
    """

    AtomIndex = []
    Hmol = Chem.RemoveHs(mol)
    for atom in Hmol.GetAtoms():
        temp = []
        if atom.GetAtomicNum() == 6:
            for neighatom in atom.GetNeighbors():
                if neighatom.GetAtomicNum() == 6:
                    temp.append(0)
                elif neighatom.GetAtomicNum() == 1:
                    continue
                else:
                    temp.append(1)
            if sum(temp) == 0:
                AtomIndex.append(atom.GetIdx())

    return AtomIndex


###################################
def FormCATSLabel(PathLength=10):

    """
    #################################################################
    Construct the CATS label such as AA0, AA1,....AP3,.......
    The result is a list format.
    A   acceptor;
    P   positive;
    N   negative;
    L   lipophilic;
    D   donor;
    #################################################################
    """
    AtomPair = [
        "DD",
        "DA",
        "DP",
        "DN",
        "DL",
        "AA",
        "AP",
        "AN",
        "AL",
        "PP",
        "PN",
        "PL",
        "NN",
        "NL",
        "LL",
    ]
    CATSLabel = []
    for i in AtomPair:
        for k in range(PathLength):
            CATSLabel.append("CATS_" + i + str(k))
    return CATSLabel


###################################


def FormCATSDict(AtomDict, CATSLabel):
    """
    #################################################################
    Construt the CATS dict.
    The result is a dict format.
    #################################################################
    """

    temp = []
    for i in AtomDict:
        for j in AtomDict[i]:
            if len(j) == 0:
                continue
            else:
                temp.append(j + str(i))

    res = dict()
    for i in set(temp):
        res.update({"CATS_" + i: temp.count(i)})

    result = dict(zip(CATSLabel, [0 for i in CATSLabel]))
    result.update(res)

    return result


###################################


def AssignAtomType(mol):
    """
    #################################################################
    Assign the atoms in the mol object into each of the PPP type
    according to PPP list definition.
    Note: res is a dict form such as {'A': [2], 'P': [], 'N': [4]}
    #################################################################
    """
    res = dict()
    for ppptype in PPP:
        temp = []
        for i in PPP[ppptype]:
            patt = Chem.MolFromSmarts(i)
            atomindex = mol.GetSubstructMatches(patt)
            atomindex = [k[0] for k in atomindex]
            temp.extend(atomindex)
        res.update({ppptype: temp})
    temp = ContructLFromGraphSearch(mol)
    temp.extend(res["L"])
    res.update({"L": temp})

    return res


###################################
def CATS2D(mol, PathLength=10, scale=3):
    """
    #################################################################
    The main program for calculating the CATS descriptors.
    CATS: chemically advanced template serach
    ----> CATS_DA0 ....
    Usage:
        result=CATS2D(mol,PathLength = 10,scale = 1)
        Input: mol is a molecule object.
               PathLength is the max topological distance between two atoms.
               scale is the normalization method (descriptor scaling method)
               scale = 1 indicates that no normalization. That is to say: the
               values of the vector represent raw counts ("counts").
               scale = 2 indicates that division by the number of non-hydrogen
               atoms (heavy atoms) in the molecule.
               scale = 3 indicates that division of each of 15 possible PPP pairs
               by the added occurrences of the two respective PPPs.
        Output: result is a dict format with the definitions of each descritor.
    #################################################################
    """
    Hmol = Chem.RemoveHs(mol)
    AtomNum = Hmol.GetNumAtoms()
    atomtype = AssignAtomType(Hmol)
    DistanceMatrix = Chem.GetDistanceMatrix(Hmol)
    DM = scipy.triu(DistanceMatrix)
    tempdict = {}
    for PL in range(0, PathLength):
        if PL == 0:
            Index = [[k, k] for k in range(AtomNum)]
        else:
            Index1 = scipy.argwhere(DM == PL)
            Index = [[k[0], k[1]] for k in Index1]
        temp = []
        for j in Index:
            temp.extend(MatchAtomType(j, atomtype))
        tempdict.update({PL: temp})

    CATSLabel = FormCATSLabel(PathLength)
    CATS1 = FormCATSDict(tempdict, CATSLabel)

    ####set normalization 3
    AtomPair = [
        "DD",
        "DA",
        "DP",
        "DN",
        "DL",
        "AA",
        "AP",
        "AN",
        "AL",
        "PP",
        "PN",
        "PL",
        "NN",
        "NL",
        "LL",
    ]
    temp = []
    for i, j in tempdict.items():
        temp.extend(j)

    AtomPairNum = {}
    for i in AtomPair:
        AtomPairNum.update({i: temp.count(i)})
    ############################################
    CATS = {}
    if scale == 1:
        CATS = CATS1
    if scale == 2:
        for i in CATS1:
            CATS.update({i: round(CATS1[i] / (AtomNum + 0.0), 3)})
    if scale == 3:
        for i in CATS1:
            if AtomPairNum[i[5:7]] == 0:
                CATS.update({i: round(CATS1[i], 3)})
            else:
                CATS.update({i: round(CATS1[i] / (AtomPairNum[i[5:7]] + 0.0), 3)})

    return CATS


import numpy as np

def CalcCATS(mol, **kwargs):
    map = CATS2D(mol)
    res = np.array([map[key] for key in sorted(map.keys())])
    return res


from rdkit import Chem
from rdkit.Chem import GraphDescriptors as desc


class CalcKappa:
    """
    #################################################################
    Calculation of molecular shape index for one bonded fragment

    ---->kappa1

    Usage:

        result=Kappa1(mol)

        Input: mol is a molecule object.

        Output: result is a numeric value.
    #################################################################
    """
    def __init__(self, is_alpha=False, n_bond=1):
        self.is_alpha = is_alpha
        self.n_bond = n_bond

    def __call__(self, mol, **kwargs):
        alpha = desc.HallKierAlpha(mol) if self.is_alpha else 0.0
        # P = mol.GetNumBonds(onlyHeavy=1) + alpha
        P = len(Chem.FindAllPathsOfLengthN(mol, self.n_bond)) + alpha
        A = mol.GetNumHeavyAtoms() + alpha + 1 - self.n_bond
        denom = P + alpha
        if denom:
            if A % 2 == 1 and self.n_bond == 3:
                kappa = (A) * (A + 1) ** 2 / denom ** 2
            else:
                kappa = (A) * (A - 1) ** 2 / denom ** 2
        else:
            kappa = 0.0
        return round(kappa, 3)


def Flexibility(mol):
    """
    #################################################################
    Calculation of Kier molecular flexibility index

    ---->phi

    Usage:

        result=Flexibility(mol)

        Input: mol is a molecule object.

        Output: result is a numeric value.
    #################################################################
    """
    kappa1 = CalcKappa(is_alpha=True, n_bond=1)(mol)
    kappa2 = CalcKappa(is_alpha=True, n_bond=1)(mol)
    A = mol.GetNumHeavyAtoms()
    phi = kappa1 * kappa2 / (A + 0.0)

    return round(phi, 3)



# -*- coding: utf-8 -*-
#  Copyright (c) 2016-2017, Zhijiang Yao, Jie Dong and Dongsheng Cao
#  All rights reserved.
#  This file is part of the PyBioMed.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the PyBioMed source tree.
"""
##############################################################################
The calculation of some commonly used basak information index  based on its
topological structure. You can get 21 molecular connectivity descriptors.
You can freely use and distribute it. If you hava  any problem, you could
contact with us timely!
Authors: Zhijiang Yao and Dongsheng Cao.
Date: 2016.06.04
Email: gadsby@163.
##############################################################################
"""


# Core Library modules
import copy

# Third party modules
import numpy
from rdkit import Chem

Version = 1.0

############################################################################


def _CalcEntropy(Probability):
    """
    #################################################################
    **Internal used only**
    Calculation of entropy (Information content) for probability given
    #################################################################
    """
    res = 0.0
    for i in Probability:
        if i != 0:
            res = res - i * numpy.log2(i)

    return res


def CalcBasakIC0(mol, **kwargs):

    """
    #################################################################
    Obtain the information content with order 0 proposed by Basak
    ---->IC0
    #################################################################
    """

    BasakIC = 0.0
    Hmol = Chem.AddHs(mol)
    nAtoms = Hmol.GetNumAtoms()
    IC = []
    for i in range(nAtoms):
        at = Hmol.GetAtomWithIdx(i)
        IC.append(at.GetAtomicNum())
    Unique = numpy.unique(IC)
    NAtomType = len(Unique)
    NTAtomType = numpy.zeros(NAtomType, numpy.float)
    for i in range(NAtomType):
        NTAtomType[i] = IC.count(Unique[i])

    if nAtoms != 0:
        # print sum(NTAtomType/nAtoms)
        BasakIC = _CalcEntropy(NTAtomType / nAtoms)
    else:
        BasakIC = 0.0

    return BasakIC


def CalcBasakSIC0(mol, **kwargs):
    """
    #################################################################
    Obtain the structural information content with order 0
    proposed by Basak
    ---->SIC0
    #################################################################
    """

    Hmol = Chem.AddHs(mol)
    nAtoms = Hmol.GetNumAtoms()
    IC = CalcBasakIC0(mol)
    if nAtoms <= 1:
        BasakSIC = 0.0
    else:
        BasakSIC = IC / numpy.log2(nAtoms)

    return BasakSIC


def CalcBasakCIC0(mol, **kwargs):

    """
    #################################################################
    Obtain the complementary information content with order 0
    proposed by Basak
    ---->CIC0
    #################################################################
    """
    Hmol = Chem.AddHs(mol)
    nAtoms = Hmol.GetNumAtoms()
    IC = CalcBasakIC0(mol)
    if nAtoms <= 1:
        BasakCIC = 0.0
    else:
        BasakCIC = numpy.log2(nAtoms) - IC

    return BasakCIC


class CalcBasakICn:
    def __init__(self, num_path=1):
        self.num_path = num_path

    def __call__(self, mol, **kwargs):
        Hmol = Chem.AddHs(mol)
        nAtoms = Hmol.GetNumAtoms()
        TotalPath = Chem.FindAllPathsOfLengthN(Hmol, self.num_path, useBonds=0, useHs=1)
        if len(TotalPath) == 0:
            BasakIC = 0.0
        else:
            IC = {}
            for i in range(nAtoms):
                temp = []
                at = Hmol.GetAtomWithIdx(i)
                temp.append([at.GetAtomicNum()])
                for index in TotalPath:
                    if i == index[0]:
                        temp.append(
                            [Hmol.GetAtomWithIdx(kk).GetAtomicNum() for kk in index[1:]]
                        )
                    if i == index[-1]:
                        cds = list(index)
                        cds.reverse()
                        temp.append(
                            [Hmol.GetAtomWithIdx(kk).GetAtomicNum() for kk in cds[1:]]
                        )
                # print temp

                IC[str(i)] = temp
            cds = []
            for value in IC.values():
                value.sort()
                cds.append(value)
            kkk = list(range(len(cds)))
            aaa = copy.deepcopy(kkk)
            res = []
            for i in aaa:
                if i in kkk:
                    jishu = 0
                    kong = []
                    temp1 = cds[i]
                    for j in aaa:
                        if cds[j] == temp1:
                            jishu = jishu + 1
                            kong.append(j)
                    for ks in kong:
                        kkk.remove(ks)
                    res.append(jishu)

            # print res
            BasakIC = _CalcEntropy(numpy.array(res, numpy.float) / sum(res))

        return BasakIC


class CalcBasakSICn:
    def __init__(self, num_path=1):
        self.num_path = num_path

    def __call__(self, mol, **kwargs):
        Hmol = Chem.AddHs(mol)
        nAtoms = Hmol.GetNumAtoms()
        IC = CalcBasakICn(num_path=self.num_path)(mol)
        if nAtoms <= 1:
            BasakSIC = 0.0
        else:
            BasakSIC = IC / numpy.log2(nAtoms)

        return BasakSIC


class CalcBasakCICn:
    def __init__(self, num_path=1):
        self.num_path = num_path

    def __call__(self, mol, **kwargs):
        Hmol = Chem.AddHs(mol)
        nAtoms = Hmol.GetNumAtoms()
        IC = CalcBasakICn(num_path=self.num_path)(mol)
        if nAtoms <= 1:
            BasakCIC = 0.0
        else:
            BasakCIC = numpy.log2(nAtoms) - IC

        return BasakCIC




from rdkit import Chem
from rdkit.Chem import rdchem
import numpy as np
from rdkit.Chem import GraphDescriptors as graph


class Chinp:
    """
    #################################################################
    Calculation of molecular connectivity chi index for path order n
    #################################################################
    """
    def __init__(self, n_path=2):
        self.n_path = n_path

    def __call__(self, mol, **kwargs):
        accum = 0.0
        if self.n_path in [0, 1]:
            if self.n_path == 0:
                deltas = [x.GetDegree() for x in mol.GetAtoms()]
            else:
                deltas = [x.GetBeginAtom().GetDegree() * x.GetEndAtom().GetDegree() for x in mol.GetBonds()]
            while 0 in deltas:
                deltas.remove(0)
            deltas = np.array(deltas, 'd')
            accum = sum(np.sqrt(1. / deltas))
        else:
            deltas = [x.GetDegree() for x in mol.GetAtoms()]
            for path in Chem.FindAllPathsOfLengthN(mol, self.n_path + 1, useBonds=0):
                cAccum = 1.0
                for idx in path:
                    cAccum *= deltas[idx]
                if cAccum:
                    accum += 1. / np.sqrt(cAccum)
        return accum


class Chinch:
    """
    #################################################################
    Calculation of molecular connectivity chi index for cycles of n
    #################################################################
    """
    def __init__(self, n_cycle=3):
        self.n_cycle = n_cycle

    def __call__(self, mol, **kwargs):
        accum = 0.0
        deltas = [x.GetDegree() for x in mol.GetAtoms()]
        for tup in mol.GetRingInfo().AtomRings():
            cAccum = 1.0
            if len(tup) == self.n_cycle:
                for idx in tup:
                    cAccum *= deltas[idx]
                if cAccum:
                    accum += 1. / np.sqrt(cAccum)

        return accum


def MeanRandic(mol):
    """
    #################################################################
    Calculation of mean chi1 (Randic) connectivity index.

    ---->mchi1

    Usage:

        result=MeanRandic(mol)

        Input: mol is a molecule object.

        Output: result is a numeric value
    #################################################################
    """
    cc = [x.GetBeginAtom().GetDegree() * x.GetEndAtom().GetDegree() for x in mol.GetBonds()]
    while 0 in cc:
        cc.remove(0)
    cc = np.array(cc, 'd')
    res = np.mean(np.sqrt(1. / cc))

    return res


class Chinc:
    def __init__(self, tag='3', is_hk=False):
        patts = {'3': '*~*(~*)~*',
                 '4': '*~*(~*)(~*)~*',
                 '4p': '*~*(~*)~*~*',
                 }
        self.patt = Chem.MolFromSmarts(patts[tag])
        self.is_hk = is_hk

    def __call__(self, mol, **kwargs):
        accum = 0.0
        deltas = [x.GetDegree() for x in mol.GetAtoms()]
        HPatt = mol.GetSubstructMatches(self.patt)
        for cluster in HPatt:
            if self.is_hk:
                deltas = [_AtomHKDeltas(mol.GetAtomWithIdx(x)) for x in cluster]
            else:
                deltas = [mol.GetAtomWithIdx(x).GetDegree() for x in cluster]
            while 0 in deltas:
                deltas.remove(0)
            if deltas != []:
                deltas1 = np.array(deltas, np.float)
                accum = accum + 1. / np.sqrt(deltas1.prod())
        return accum


class Chivnp:
    """#################################################################
    Calculation of valence molecular connectivity chi index for path order n
    #################################################################
    """
    def __init__(self, n_path):
        self.n_path = n_path

    def __call__(self, mol, **kwargs):
        deltas = graph._hkDeltas(mol, skipHs=0)
        if self.n_path == 0:
            while 0 in deltas:
                deltas.remove(0)
            deltas = np.array(deltas, 'd')
            accum = sum(np.sqrt(1. / deltas))
        else:
            accum = 0.0
            for path in Chem.FindAllPathsOfLengthN(mol, self.n_path + 1, useBonds=0):
                cAccum = 1.0
                for idx in path:
                    cAccum *= deltas[idx]
                if cAccum:
                    accum += 1. / np.sqrt(cAccum)
        return accum


class Chivnch:
    """
    #################################################################
    Calculation of valence molecular connectivity chi index for cycles of n
    #################################################################
    """
    def __init__(self, n_cycle=3):
        self.n_cycle = n_cycle

    def __call__(self, mol, **kwargs):
        accum = 0.0
        deltas = graph._hkDeltas(mol, skipHs=0)
        for tup in mol.GetRingInfo().AtomRings():
            cAccum = 1.0
            if len(tup) == self.n_cycle:
                for idx in tup:
                    cAccum *= deltas[idx]
                if cAccum:
                    accum += 1. / np.sqrt(cAccum)

        return accum


class DeltaChi:
    def __init__(self, fn1, fn2):
        self.fn1 = fn1
        self.fn2 = fn2

    def __call__(self, mol, **kwargs):
        return abs(self.fn1(mol) - self.fn2(mol))


def _AtomHKDeltas(atom, skipHs=0):
    """
    #################################################################
    Calculation of modified delta value for a molecule
    #################################################################
    """
    res = []
    n = atom.GetAtomicNum()
    if n > 1:
        nV = Chem.GetPeriodicTable().GetNOuterElecs(n)
        nHs = atom.GetTotalNumHs()
        if n < 10:
            res.append(float(nV - nHs))
        else:
            res.append(float(nV - nHs) / float(n - nV - 1))
    elif not skipHs:
        res.append(0.0)
    return res




from rdkit import Chem
from rdkit.Chem import rdPartialCharges as GMCharge

import numpy as np

iter_step = 12


class CalcElementCharge:
    def __init__(self, num_atom=6, method=np.min):
        self.n_atom = num_atom
        self.method = method

    def __call__(self, mol, **kwargs):
        Hmol = Chem.AddHs(mol)
        GMCharge.ComputeGasteigerCharges(Hmol, iter_step)
        res = []
        for atom in Hmol.GetAtoms():
            if atom.GetAtomicNum() == self.n_atom or self.n_atom == 0:
                res.append(float(atom.GetProp('_GasteigerCharge')))
        if res == []:
            return 0
        else:
            return self.method(np.array(res))


def sqsum(arr):
    return np.sum(np.square(arr))


def abs_sum(arr):
    return np.sum(np.absolute(arr))


def abs_mean(arr):
    return np.mean(np.absolute(arr))


def relp_sum(arr):
    return max(arr) / sum(arr[arr > 0])


def reln_sum(arr):
    return max(arr) / sum(arr[arr < 0])


def p_sum(arr):
    return sum(arr[arr > 0])


def p_mean(arr):
    return np.mean(arr[arr > 0])


def n_sum(arr):
    return sum(arr[arr < 0])


def n_mean(arr):
    return np.mean(arr[arr < 0])


def CalcLocalDipoleIndex(mol):
    """
    #################################################################
    Calculation of local dipole index (D)

    -->LDI

    Usage:

        result=CalculateLocalDipoleIndex(mol)

        Input: mol is a molecule object.

        Output: result is a numeric value.
    #################################################################
    """

    GMCharge.ComputeGasteigerCharges(mol, iter_step)
    res = []
    for atom in mol.GetAtoms():
        res.append(float(atom.GetProp('_GasteigerCharge')))
    cc = [np.absolute(res[x.GetBeginAtom().GetIdx()] - res[x.GetEndAtom().GetIdx()]) for x in mol.GetBonds()]
    B = len(mol.GetBonds())

    return round(sum(cc) / B, 3)


def CalcSubmolPolarityPara(mol):
    """
    #################################################################
    Calculation of submolecular polarity parameter(SPP)

    -->SPP

    Usage:

        result=CalculateSubmolPolarityPara(mol)

        Input: mol is a molecule object.

        Output: result is a numeric value.
    #################################################################
    """
    maxP = CalcElementCharge(0, np.max)
    maxN = CalcElementCharge(0, np.min)
    return round(maxP(mol) - maxN(mol), 3)





#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: Zhen Chen(chenzhen-win2009@163.com; zhenchen625@hotmail.com)
# Description: iFeatureOmega is an integrative platform for the prediction/feature engineering, visualization and analysis of features from molecular sequence, structural and ligand data sets.

from setuptools import setup, find_packages

setup(
    name = 'iFeatureOmegaCLI',
    version = '1.0.2',
    keywords='iFeatureOmegaCLI',
    description = 'An integrative platform for the prediction/feature engineering, visualization and analysis of features from molecular sequence, structural and ligand data sets',
    license = 'MIT License',
    url = 'https://github.com/Superzchen/iFeatureOmega-CLI',
    author = 'SuperZhen',
    author_email = 'chenzhen-win2009@163.com',    
    packages=find_packages("src"),
    package_dir = {'':'src'},
    
    package_data = {
        # 任何包中含有.txt文件，都包含它
        '': ['*.txt'],
        # 包含demo包data文件夹中的 *.dat文件
        'iFeatureOmegaCLI': ['data/*.txt', 'data/*.data', 'data_examples/*.csv', 'data_examples/*.txt', 'data_examples/*.pdb', 'parameters/*.json'],
    },
    platforms = 'any',
    install_requires = [
        'networkx',        
        'numpy>=1.21.4',
        'pandas>=1.3.4',        
        'scikit-learn>=1.0.1',
        'scipy>=1.7.3',
        'matplotlib==3.4.3'
        ],     
)


from rdkit.Chem import Crippen
from rdkit.Chem import MolSurf as MS
import math


class Property:
    def __init__(self, keys):
        self.keys = keys
        self.props = {
            'LogP': Crippen.MolLogP,
            'MR': Crippen.MolMR,
            'LabuteASA': MS.pyLabuteASA,
            'TPSA': MS.TPSA,
            'Hy': CalculateHydrophilicityFactor,
            'UI': CalculateUnsaturationIndex
        }

    def __call__(self, mol):
        return self.props[self.keys](mol)


def CalculateUnsaturationIndex(mol):
    """
    #################################################################
    Calculation of unsaturation index.

    ---->UI

    Usage:

        result=CalculateUnsaturationIndex(mol)

        Input: mol is a molecule object.

        Output: result is a numeric value.
    #################################################################
    """
    nd = sum([1 if b.GetBondType() == 2 else 0 for b in mol.GetBonds()])
    nt = sum([1 if b.GetBondType() == 3 else 0 for b in mol.GetBonds()])
    na = sum([1 if b.GetBondType() == 12 else 0 for b in mol.GetBonds()])
    res = math.log((1 + nd + nt + na), 2)

    return round(res, 3)


def CalculateHydrophilicityFactor(mol):
    """
    #################################################################
    Calculation of hydrophilicity factor. The hydrophilicity

    index is described in more detail on page 225 of the

    Handbook of Molecular Descriptors (Todeschini and Consonni 2000).

    ---->Hy

    Usage:

        result=CalculateHydrophilicityFactor(mol)

        Input: mol is a molecule object.

        Output: result is a numeric value.
    #################################################################
    """
    nheavy = mol.GetNumAtoms(onlyHeavy=1)
    nc = 0
    for atom in mol.GetAtoms():
        if atom.GetAtomicNum() == 6:
            nc = nc + 1
    nhy = 0
    for atom in mol.GetAtoms():
        if atom.GetAtomicNum() == 7 or atom.GetAtomicNum() == 8 or atom.GetAtomicNum() == 16:
            atomn = atom.GetNeighbors()
            for i in atomn:
                if i.GetAtomicNum() == 1:
                    nhy = nhy + 1

    res = (1 + nhy) * math.log((1 + nhy), 2) + nc * (1.0 / nheavy * math.log(1.0 / nheavy, 2)) + math.sqrt(
        (nhy + 0.0) / (nheavy ^ 2))
    return round(res, 3)






from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors as desc
import numpy as np


class PathsOfLengthN:
    def __init__(self, n):
        self.n = n

    def __call__(self, mol, **kwargs):
        paths = Chem.FindAllPathsOfLengthN(mol, self.n)
        return len(paths)


class TotalAtom:
    def __init__(self, is_weight=False):
        self.is_weight = is_weight

    def __call__(self, mol, **kwargs):
        mol = Chem.AddHs(mol)
        n_atoms = len(mol.GetAtoms())
        if self.is_weight:
            return desc.ExactMolWt(mol) / n_atoms
        return n_atoms


class FragCounter:
    def __init__(self, element: str) -> None:
        """
        Args:
            element: element to count within a molecule
        """
        self.element = element

    def __call__(self, mol, **kwargs):
        """
        Count the number of atoms of a given type.
        Args:
            mol: molecule
        Returns:
            The number of atoms of the given type.
        """
        # if the molecule contains H atoms, they may be implicit, so add them
        sub = Chem.MolFromSmarts(self.element)
        if self.element == '[H]':
            mol = Chem.AddHs(mol)
        score = len(mol.GetSubstructMatches(sub))
        return score

